! Program to obtain boundary layer depth from balloon soundings.
! Uses soundings from the University of Wyoming site.
! Can analyze multiple days by cating monthly data into the input file, allSoundings.txt.
! Boundary layer depth is from potential temperature return to surface value,
! or from the first mininum of the dew point temperature depression above the surface measurements.
! Written by Pat Arnott on a whim for a quick answer to the question.
! Modified starting on 30 Jan 2019 to obtain data from various pressure levels, especially stratospheric conditions.


!    PROGRAM MakeTimeSeriesOfPressureLevels
	IMPLICIT NONE 
	CHARACTER*200 LineIn(1000000)
	CHARACTER*200 OneLine
	REAL dataIn(1000000,11),DewDep(1000000)
	REAL PBLH, PWat,Theta_MML ! To be read from sounding.
	REAL RH_700mb, RH_500mb, RH_400mb, RH_300mb, ThetaE
	REAL H(20), P(20), T(20), Tdew(20), WindDir(20), WindSpeed(20), Theta(20) ! Properties from various levels, eg. 100 mb, etc. 
	INTEGER numP, iP ! Pressure levels to get properties of the atmoshere at.
	REAL criteria1, criteria2,height
	CHARACTER*8 height_text
	CHARACTER*2 mm,dd
	CHARACTER*4 yyyy
	CHARACTER*3 mmm  ! Month read in from the file directly.
	CHARACTER*3 timeChar ! UTC time of the sounding.
	INTEGER i,imax,id,j,is,ie,iPBLH,jEnd,k
	REAL ThetaStart, P_PBLH
	LOGICAL crit1, crit2,SoundingEnd
	INTEGER reason

! Define pressure levels that data will be obtained from.
	P(1)=1000.0
	P(2)=850.0
	P(3)=700.0
	P(4)=500.0
	P(5)=300.0
	P(6)=250.0
	P(7)=100.0
	P(8)=70.0
	P(9)=50.0
	P(10)=30.0
	P(11)=20.0
	numP=11
	DO iP=1,numP ! Initialize all variables to the no data value -99.99.
	 T(iP)=-99.99 ; Tdew(iP)=-99.99 ; WindDir(iP)=-99.99 ; WindSpeed(iP)=-99.99 ; Theta(iP)=-99.99
	ENDDO


	OPEN(1,FILE='AllSoundingsBarrow.txt')  ! input file.
	OPEN(2,FILE='SoundingTimeSeriesOutput.txt') ! output file. Change Theta to DewPtDep.
	WRITE(2,*) "mmddyyyy_UTC Time_UTC 
     * T_1000mb_C T_850mb T_700mb T_500mb T_300mb T_250mb T_100mb T_70mb T_50mb T_30mb T_20mb
     * Tdew_1000mb_C Tdew_850mb Tdew_700mb Tdew_500mb Tdew_300mb Tdew_250mb Tdew_100mb Tdew_70mb Tdew_50mb Tdew_30mb Tdew_20mb 
     * Wdir_1000mb_deg Wdir_850mb Wdir_700mb Wdir_500mb Wdir_300mb Wdir_250mb Wdir_100mb Wdir_70mb Wdir_50mb Wdir_30mb Wdir_20mb 
     * Wspd_1000mb_knots Wspd_850mb Wspd_700mb Wspd_500mb Wspd_300mb Wspd_250mb Wspd_100mb Wspd_70mb Wspd_50mb Wspd_30mb Wspd_20mb 
     * potT_1000mb_K potT_850mb potT_700mb potT_500mb potT_300mb potT_250mb potT_100mb potT_70mb potT_50mb potT_30mb potT_20mb
     * H_1000mb_m H_850mb H_700mb H_500mb H_300mb H_250mb H_100mb H_70mb H_50mb H_30mb H_20mb PrecipH20_mm"  ! File Header.

	DO i=1,1000000
	 READ(1,10,END=20) LineIn(i)
!	 WRITE(*,*) LineIn(i)
	ENDDO

10	FORMAT(a200)

20	imax=i
	CLOSE(1)

	DO i=1,imax
	OneLine=LineIn(i)
!	WRITE(*,*) OneLine(1:5),'  ',OneLine(32:35)
!	  IF ((OneLine(1:5).EQ."72489").AND.(OneLine(32:35).EQ."00Z")) THEN 
 	  IF (OneLine(1:5).EQ."70026") THEN 
		WRITE(*,*) "Found the start of the data for a sounding"
	        WRITE(*,10) OneLine
	        dd=OneLine(39:40) ! This will be specific for each location.
	        mmm=OneLine(42:44) ! Specific for each location.
		timeChar=OneLine(35:37) ! Time in UTC Specific for each location. 
		WRITE(*,*) "timeChar = " ; WRITE(*,*) timeChar
	        CALL ConvertMonth(mmm,mm)
	        yyyy=OneLine(46:49)
                DO j=i+1,i+1000  ! Find the start and end lines of the sounding.
	           OneLine=LineIn(j)
	           IF (OneLine(1:12).EQ."Precipitable") THEN 
	                DO k=1,44 ; Oneline(k:k)=' ' ; ENDDO
	                READ(OneLine,*) Pwat
	                jEnd = j
	                GOTO 210
	 	  ENDIF
		ENDDO
210		WRITE(*,*) 'Line of End of Sounding = ',jEnd
		DO j=i+1,jEnd
		   OneLine=LineIn(j)
	           IF (OneLine(6:43).EQ."Mean mixed layer potential temperature") THEN
			DO k=1,44 ; Oneline(k:k)=' ' ; ENDDO
	               	READ(OneLine,*) Theta_MML
		   ENDIF
	        ENDDO 

		SoundingEnd=.TRUE.
		DO j=i+1,jEnd  ! Find the start and end lines of the sounding.
	           OneLine=LineIn(j)
	           IF ((OneLine(5:7).EQ."hPa").AND.SoundingEnd) THEN ! Specific for each sounding.
			is=j+2
			SoundingEnd=.FALSE.
	 	   END IF
                   IF (OneLine(1:7).EQ."Station") ie=j-1 ! Specific for each sounding.
	        ENDDO
		WRITE(*,*) 'is= ',is,'ie= ',ie
		RH_700mb=-99.99 ; RH_500mb=-99.99 ; RH_400mb=-99.99 ; RH_300mb=-99.99 ; ThetaE=-99.99 ; iPBLH=-99
		DO j=is,ie  ! Read in the sounding.
	           OneLine=LineIn(j)
	           READ(OneLine,*,iostat=reason) (dataIn(j,id),id=1,11)
		     WRITE(*,*) 'reason = ' ; WRITE(*,*) reason
		     WRITE(*,*) OneLine
		   IF (reason.EQ.0) THEN
		     DewDep(j) = dataIn(j,3) - dataIn(j,4)
		     IF (abs(dataIn(j,1)-700.0).LE.0.2) RH_700mb=dataIn(j,5)
		     IF (abs(dataIn(j,1)-500.0).LE.0.2) RH_500mb=dataIn(j,5)	
		     IF (abs(dataIn(j,1)-400.0).LE.0.2) RH_400mb=dataIn(j,5)	
		     IF (abs(dataIn(j,1)-300.0).LE.0.2) RH_300mb=dataIn(j,5)
		     DO iP = 1,numP ! Extract atmospheric properties for chosen pressure levels.
	 	       IF (abs(dataIn(j,1)-P(iP)).LE.0.2) THEN
			  T(iP)=dataIn(j,3) ; Tdew(iP)=dataIn(j,4) ; Winddir(iP)=dataIn(j,7) ; Windspeed(iP)=dataIn(j,8) 
			  Theta(iP)=dataIn(j,9) ; H(iP)=dataIn(j,2)
		       ENDIF
		     ENDDO
	           ELSE
	             ie=j-1
	             GOTO 2000
	           ENDIF	
	        ENDDO 
2000	        ThetaStart = (dataIn(is,9)+dataIn(is+1,9))/2.0 ! Surface potential T.
  		DO j=is+6,ie  ! Move about 4 measurements above the first measurement for starting the hunt for the boundary layer height.
		   criteria1 =  (dataIn(j,9)-ThetaStart) ! Criteria based on potential temperature increasing back to surface value.
		   crit1 = (criteria1.GE.0.0)
		   criteria2 =  (DewDep(j) - DewDep(j-1)) + (DewDep(j) - DewDep(j+1))  ! Criteria based on finding relative minimum of DewPt Depression.
	           crit2 = (DewDep(j).LT.DewDep(j-1)).AND.(DewDep(j).LT.DewDep(j+1))
!	           WRITE(8,*) mm,'/',dd,'/',yyyy,' ',dataIn(j,1),dataIn(j,2),dataIn(j,9),criteria1,DewDep(j),criteria2  ! Write out the pressure and boundary layer height determination criteria. 
!		   IF (crit2) THEN  ! Using dew point depression minimum.
		   IF (crit1) THEN ! Using potential temperature change.
		     iPBLH=j
	             GOTO 100
	           ENDIF
		ENDDO
100	        PBLH=dataIn(iPBLH,2)/1000.0 -1.422  ! Boundary layer height in km AGL.
	        ThetaE = dataIn(iPBLH,10)
		P_PBLH=dataIn(iPBLH,1) ! Pressure at the top of the boundary layer.
  	IF (timeChar.EQ.'00Z') THEN
	 WRITE(2,*) mm,'/',dd,'/',yyyy,' ','00:00:00',' ',(T(iP),iP=1,numP),
     *    (Tdew(iP),iP=1,numP),
     *    (WindDir(iP),iP=1,numP),
     *    (WindSpeed(iP),iP=1,numP),
     *    (Theta(iP),iP=1,numP), (H(iP),iP=1,numP), PWat
	ENDIF 
  	IF (timeChar.EQ.'06Z') THEN
	 WRITE(2,*) mm,'/',dd,'/',yyyy,' ','06:00:00',' ',(T(iP),iP=1,numP),
     *    (Tdew(iP),iP=1,numP),
     *    (WindDir(iP),iP=1,numP),
     *    (WindSpeed(iP),iP=1,numP),
     *    (Theta(iP),iP=1,numP), (H(iP),iP=1,numP), PWat
	ENDIF 
  	IF (timeChar.EQ.'12Z') THEN
	 WRITE(2,*) mm,'/',dd,'/',yyyy,' ','12:00:00',' ',(T(iP),iP=1,numP),
     *    (Tdew(iP),iP=1,numP),
     *    (WindDir(iP),iP=1,numP),
     *    (WindSpeed(iP),iP=1,numP),
     *    (Theta(iP),iP=1,numP), (H(iP),iP=1,numP), PWat
	ENDIF 
  	IF (timeChar.EQ.'18Z') THEN
	 WRITE(2,*) mm,'/',dd,'/',yyyy,' ','18:00:00',' ',(T(iP),iP=1,numP),
     *    (Tdew(iP),iP=1,numP),
     *    (WindDir(iP),iP=1,numP),
     *    (WindSpeed(iP),iP=1,numP),
     *    (Theta(iP),iP=1,numP), (H(iP),iP=1,numP), PWat
	ENDIF 

       END IF
	  
	ENDDO
	   

	CLOSE(2)


	END
	
	SUBROUTINE ConvertMonth(mmm,mm)
	IMPLICIT NONE
	CHARACTER*3 mmm
	CHARACTER*2 mm
	
	IF (mmm.EQ."Jan") mm="01"
	IF (mmm.EQ."Feb") mm="02"
	IF (mmm.EQ."Mar") mm="03"
	IF (mmm.EQ."Apr") mm="04"	
	IF (mmm.EQ."May") mm="05"
	IF (mmm.EQ."Jun") mm="06"
	IF (mmm.EQ."Jul") mm="07"
	IF (mmm.EQ."Aug") mm="08"	
	IF (mmm.EQ."Sep") mm="09"
	IF (mmm.EQ."Oct") mm="10"
	IF (mmm.EQ."Nov") mm="11"
	IF (mmm.EQ."Dec") mm="12"	
	
	RETURN
	END




